// Menu.cpp
// Neuron Computing Inc.
#include "Toolkit.h"

#ifdef MOTIF
	#include <Xm/Separator.h>
	#include <Xm/CascadeB.h>
#else
	#include <stdio.h>
#endif

extern XApp* g_app;

// Action callback
void component_actionPerformed(Widget w, XtPointer client_data, XtPointer call_data)
{
	g_app->m_mainFrame->actionPerformed((int)client_data + 1);
}

//////////////////////////////////////////////////////////////////////
// Utility functions for popup menus for ATHENA:

#ifndef MOTIF

static int x_location = 350;
static int y_location = 300;

static int  poped_up = 0;

//#define XtPointer caddr_t

Widget popup_shell, popup_shell2;

static int local_count_callbacks = 1;

typedef XtActionsRec aXtActionsRec;

void set_leave_widget_callback(Widget w, XtActionProc cb)
{
	static char buf[100], name[20];
	sprintf(name,"name00%d",local_count_callbacks++);
	sprintf(buf,"<LeaveNotify>: %s()",name);
	static aXtActionsRec myActions[1];
	myActions[0].proc = (XtActionProc)cb;
	myActions[0].string = name;
	XtAddActions(myActions,1);
	XtOverrideTranslations(w,XtParseTranslationTable(buf));
}

void set_enter_widget_callback(Widget w, XtActionProc cb)
{
	static char buf[100], name[20];
	sprintf(name,"name00%d",local_count_callbacks++);
	sprintf(buf,"<EnterNotify>: %s()",name);
	static aXtActionsRec myActions[1];
	myActions[0].proc = (XtActionProc)cb;
	myActions[0].string = name;
	XtAddActions(myActions,1);
	XtOverrideTranslations(w,XtParseTranslationTable(buf));
}

void menu_pane_chosen(Widget w, XtPointer client_data, XtPointer)
{ 
	int selection_number = (int) client_data;
	component_actionPerformed(w, (XtPointer)selection_number, 0);
	XtPopdown(popup_shell);
}

void do_popdown(Widget, caddr_t, caddr_t)
{
	XtPopdown(popup_shell);
	poped_up = 0;
}

void makeMenu(Widget topLevel, char **item_names, int num_items)
{
	int i;
	Widget menubox, a_form, menupane[10];
	Arg args[10];
	unsigned int key_buttons;
	int win_x, win_y;
	Window root, child;

	if (poped_up == 1)
		return;
	poped_up = 1;

	XQueryPointer(XtDisplay(topLevel),XtWindow(topLevel),
					&root, &child,
					&x_location, &y_location, &win_x, &win_y,
					&key_buttons);

	// Throw away all mouse movement events so the popup menu
	// does not popdown due to residual mouse motion:
	XSync(XtDisplay(topLevel), False);

	i=0;
	XtSetArg(args[i],XtNx,x_location-40); i++;
	XtSetArg(args[i],XtNy,y_location-40); i++;
	XtSetArg(args[i],XtNborderWidth,4); i++;

	popup_shell = XtCreatePopupShell("popup_shell", overrideShellWidgetClass,
										topLevel, args, i);

	i=0;
	XtSetArg(args[i],XtNborderWidth,1); i++;

	a_form = XtCreateManagedWidget("a_form", formWidgetClass,
									popup_shell, args, i);

	menubox = XtCreateManagedWidget("menubox", boxWidgetClass,
									a_form, args, i);

	for (i = 0; i < num_items; i++)
	{
		int j = 0;
		XtSetArg(args[j],XtNborderWidth,0); j++;
		XtSetValues(popup_shell,args,j);
		menupane[i] = XtCreateManagedWidget(item_names[i],
											COMMAND, menubox,
											args, j);
		XtAddCallback(menupane[i], XtNcallback,
						(XtCallbackProc)menu_pane_chosen, (XtPointer)i);
    }
	set_leave_widget_callback(popup_shell, (XtActionProc)do_popdown);

	XtPopup(popup_shell,XtGrabNone);

	// Warp pointer to edit field
	XWarpPointer(XtDisplay(popup_shell),None,XtWindow(popup_shell),
					0, 0, 0, 0, 40, 40);
	XSync(XtDisplay(topLevel),False);
}

// Menu callback
void do_make_menu(Widget w, XApp* app, XtPointer call_data)
{
	makeMenu(app->m_mainFrame->m_w, app->m_menu_items, app->m_menu_item_count);
}

#else

// MOTIF

Menu::Menu()
{
	m_menu = NULL;
}

void Menu::create(Widget menubar, char* name)
{
	Arg args[4];
	int n=0;
	m_menu = XmCreatePulldownMenu(menubar, "Menu1", NULL, 0);

	XtSetArg(args[n], XmNmnemonic, name[0]); n++;
	XtSetArg(args[n], XmNsubMenuId, m_menu); n++;

	Widget cascade = XmCreateCascadeButton(menubar, name, args, n);
	XtManageChild(cascade);
}

void Menu::addMenuItems(char **item_names, int num_items)
{
	// WidgetList menuItems = (WidgetList)XtMalloc(num_items*sizeof(Widget));
	for (int i=0;i<num_items;i++)
	{
		// menuItems[i] = XmCreateOptionMenu(the_menu, item_names[i], NULL, 0);
		Widget menuItem;
		if (item_names[i][0] == 0)
			menuItem = XmCreateSeparator(m_menu, "separator", NULL, 0);
		else
			menuItem = XtCreateWidget(item_names[i],
										xmPushButtonWidgetClass,
										m_menu, NULL, 0);

		XtAddCallback(menuItem, XmNactivateCallback,
						(XtCallbackProc)component_actionPerformed, (XtPointer)i);
		XtManageChild(menuItem);
    }
	XtManageChild(m_menu);
	// XtManageChildren(menuItems, num_items);
}

#endif

MenuBar::MenuBar()
{
	m_menu = NULL;
}

void MenuBar::create(Frame* frame)
{
	Widget parent = frame->m_layout;
#ifdef MOTIF
	m_menu = XmCreateMenuBar(parent, "menubar", NULL, 0);
#else
	Arg args[10];
	int n = 0;
	XtSetArg(args[n], XtNy, 3); n++;
	XtSetArg(args[n], XtNx, 3); n++;
	XtSetArg(args[n], XtNborderWidth, 3); n++;
	m_menu = XtCreateManagedWidget("Menu", COMMAND, parent, args, n);
	XtAddCallback(menu, XtNcallback,
					(XtCallbackProc)do_make_menu,
					(XtPointer)app);
#endif
}

void MenuBar::addMenu(char* name, char** menu_items, int menu_item_count)
{
#ifdef MOTIF
	Menu menu;
	menu.create(m_menu, name);
	menu.addMenuItems(menu_items, menu_item_count);
	XtManageChild(m_menu);
#else
#endif
}
